# -*- coding: utf-8 -*-

"""Azure gallery utils cli module."""

# Copyright (c) 2022 SUSE LLC
#
# This file is part of azure_img_utils. azure_img_utils provides an
# api and command line utilities for handling images in the Azure Cloud.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import click
import logging
import sys

from azure_img_utils.cli.cli_utils import (
    add_options,
    get_config,
    process_shared_options,
    shared_options,
    echo_style
)
from azure_img_utils.azure_image import AzureImage


# -----------------------------------------------------------------------------
# Gallery commands function
@click.group(name="gallery-image-version")
def gallery_image_version():
    """
    Commands for gallery image version management.
    """


# -----------------------------------------------------------------------------
# exists command function
@gallery_image_version.command()
@click.option(
    '--gallery-image-name',
    type=click.STRING,
    required=True,
    help='Name of the gallery image to check.'
)
@click.option(
    '--gallery-name',
    type=click.STRING,
    required=True,
    help='Name of the gallery to check image existence.'
)
@click.option(
    '--gallery-image-version',
    type=click.STRING,
    required=True,
    help='Version of the gallery image to check.'
)
@add_options(shared_options)
@click.pass_context
def exists(
    context,
    gallery_image_name,
    gallery_name,
    gallery_image_version,
    **kwargs
):
    """
    Checks if a gallery image version exists
    """

    process_shared_options(context.obj, kwargs)
    config_data = get_config(context.obj)
    logger = logging.getLogger('azure_img_utils')
    logger.setLevel(config_data.log_level)

    try:
        az_img = AzureImage(
            container=config_data.container,
            storage_account=config_data.storage_account,
            credentials_file=config_data.credentials_file,
            resource_group=config_data.resource_group,
            log_level=config_data.log_level,
            log_callback=logger
        )
        exists = az_img.gallery_image_version_exists(
            gallery_name,
            gallery_image_name,
            gallery_image_version,
            config_data.resource_group
        )

        if exists:
            echo_style('true', config_data.no_color, fg='green')
        else:
            echo_style('false', config_data.no_color)

    except Exception as e:
        echo_style(
            'Unable to check gallery image version existence',
            config_data.no_color,
            fg='red'
        )
        echo_style(str(e), config_data.no_color, fg='red')
        sys.exit(1)


# -----------------------------------------------------------------------------
# gallery image create command function
@gallery_image_version.command()
@click.option(
    '--blob-name',
    type=click.STRING,
    required=True,
    help='Name of the blob for the gallery image.'
)
@click.option(
    '--gallery-name',
    type=click.STRING,
    required=True,
    help='Name of the gallery where the image will be created.'
)
@click.option(
    '--gallery-image-name',
    type=click.STRING,
    required=True,
    help='Name of the gallery image to be created.'
)
@click.option(
    '--gallery-image-version',
    type=click.STRING,
    required=True,
    help='Version of the gallery image to create.'
)
@click.option(
    '--force-replace-image',
    is_flag=True,
    default=False,
    help='Delete the gallery image prior to create if it already exists.'
)
@add_options(shared_options)
@click.pass_context
def create(
    context,
    blob_name,
    gallery_name,
    gallery_image_name,
    gallery_image_version,
    force_replace_image,
    **kwargs
):
    """
    Creates a gallery image based on the already uploaded blob.
    """
    process_shared_options(context.obj, kwargs)
    config_data = get_config(context.obj)
    logger = logging.getLogger('azure_img_utils')
    logger.setLevel(config_data.log_level)

    try:
        az_img = AzureImage(
            container=config_data.container,
            storage_account=config_data.storage_account,
            credentials_file=config_data.credentials_file,
            resource_group=config_data.resource_group,
            log_level=config_data.log_level,
            log_callback=logger
        )
        img_name = az_img.create_gallery_image_version(
            blob_name,
            gallery_name,
            gallery_image_name,
            gallery_image_version,
            config_data.region,
            force_replace_image=force_replace_image,
            gallery_resource_group=config_data.resource_group
        )

        if img_name and config_data.log_level != logging.ERROR:
            echo_style(
                f'gallery image version {img_name} created',
                config_data.no_color,
                fg='green'
            )

    except Exception as e:
        echo_style(
            'Unable to create gallery image',
            config_data.no_color,
            fg='red'
        )
        echo_style(str(e), config_data.no_color, fg='red')
        sys.exit(1)


# -----------------------------------------------------------------------------
# gallery image delete command function
@gallery_image_version.command()
@click.option(
    '--gallery-name',
    type=click.STRING,
    required=True,
    help='Name of the gallery where the image will be deleted.'
)
@click.option(
    '--gallery-image-name',
    type=click.STRING,
    required=True,
    help='Name of the image to delete.'
)
@click.option(
    '--gallery-image-version',
    type=click.STRING,
    required=True,
    help='Version of the gallery image to delete.'
)
@add_options(shared_options)
@click.confirmation_option(
    help='This command will delete the specified gallery image. Are you sure?'
)
@click.pass_context
def delete(
    context,
    gallery_name,
    gallery_image_name,
    gallery_image_version,
    **kwargs
):
    """
    Deletes a gallery image if the image exists in the gallery
    """

    process_shared_options(context.obj, kwargs)
    config_data = get_config(context.obj)
    logger = logging.getLogger('azure_img_utils')
    logger.setLevel(config_data.log_level)

    try:
        az_img = AzureImage(
            container=config_data.container,
            storage_account=config_data.storage_account,
            credentials_file=config_data.credentials_file,
            resource_group=config_data.resource_group,
            log_level=config_data.log_level,
            log_callback=logger
        )
        az_img.delete_gallery_image_version(
            gallery_name,
            gallery_image_name,
            gallery_image_version,
            gallery_resource_group=config_data.resource_group
        )

    except Exception as e:
        echo_style(
            'Unable to delete gallery image version',
            config_data.no_color,
            fg='red'
        )
        echo_style(str(e), config_data.no_color, fg='red')
        sys.exit(1)



# -*- coding: utf-8 -*-

"""Azure image utils cli module utils."""

# Copyright (c) 2022 SUSE LLC
#
# This file is part of azure_img_utils. azure_img_utils provides an
# api and command line utilities for handling images in the Azure Cloud.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import click
import json
import logging
import os
import sys
import yaml

from collections import namedtuple, ChainMap


default_config_dir = os.path.expanduser('~/.config/azure_img_utils/')
default_profile = 'default'

config_defaults = {
    'config_dir': default_config_dir,
    'profile': default_profile,
    'log_level': logging.INFO,
    'no_color': False,
    'credentials_file': '',
    'resource_group': '',
    'storage_account': '',
    'container': '',
    'region': '',
    'publisher_id': '',
    'notification_emails': '',
}

azure_img_utils_config = namedtuple(
    'az_img_utils_config',
    sorted(config_defaults)
)

# -----------------------------------------------------------------------------
# Shared options
shared_options = [
    click.option(
        '-C',
        '--config-dir',
        type=click.Path(exists=True),
        help='Azure Image utils config directory to use. Default: '
             '~/.config/azure_img_utils/'
    ),
    click.option(
        '--profile',
        help='The configuration profile to use. Expected to match '
             'a config file in config directory. Example: production, '
             'for ~/.config/azure_img_utils/production.yaml. The default '
             'value is default: ~/.config/azure_img_utils/default.yaml'
    ),
    click.option(
        '--no-color',
        is_flag=True,
        help='Remove ANSI color and styling from output.'
    ),
    click.option(
        '--verbose',
        'log_level',
        flag_value=logging.DEBUG,
        help='Display debug level logging to console.'
    ),
    click.option(
        '--info',
        'log_level',
        flag_value=logging.INFO,
        default=True,
        help='Display logging info to console. (Default)'
    ),
    click.option(
        '--quiet',
        'log_level',
        flag_value=logging.ERROR,
        help='Display only errors to console.'
    ),
    click.option(
        '--credentials-file',
        type=click.Path(exists=True),
        help='Azure Image utils credentials file to use.'
    ),
    click.option(
        '--resource-group',
        type=click.STRING,
        help='Azure Image utils resource group to use.'
    ),
    click.option(
        '--region',
        type=click.STRING,
        help='The region to use for the image requests.'
    ),
    click.option(
        '--storage-account',
        type=click.STRING,
        help='Storage account for the blobs.'
    ),
    click.option(
        '--container',
        type=click.STRING,
        help='Container for the blob to check.'
    ),
    click.option(
        '--publisher-id',
        type=click.STRING,
        help='Id of the publisher to use for the publication.'
    ),
    click.option(
        '--notification-emails',
        type=click.STRING,
        help='Comma separated list of emails to be notified.'
             'For migrated offers this param is ignored and the notifications'
             ' will be sent to the email address set as Seller contact info '
             'section of your Account settings in Partner Center'
    )
]


def add_options(options):
    def _add_options(func):
        for option in reversed(options):
            func = option(func)
        return func
    return _add_options


# -------------------------------------------------
# Get Config
def get_config(cli_context):
    """
    Process Azure Image utils config.
    Use ChainMap to build config values based on
    command line args, config and defaults.
    """
    config_dir = cli_context['config_dir'] or default_config_dir
    profile = cli_context['profile'] or default_profile

    config_values = {}
    config_file_path = os.path.join(config_dir, profile + '.yaml')

    try:
        with open(config_file_path) as config_file:
            config_values = yaml.safe_load(config_file)
    except FileNotFoundError:
        echo_style(
            f'Config file: {config_file_path} not found. Using default '
            f'configuration values.',
            no_color=True
        )

    cli_values = {
        key: value for key, value in cli_context.items() if value is not None
    }
    data = ChainMap(cli_values, config_values, config_defaults)

    try:
        config_data = azure_img_utils_config(**data)
    except TypeError as e:
        echo_style(
            f'Found unknown keyword in config file {config_file_path}',
            no_color=True
        )
        echo_style(str(e), no_color=True)
        sys.exit(1)

    return config_data


# -----------------------------------------------------------------------------
# Printing options
def echo_style(message, no_color, fg='yellow'):
    """
    Echo stylized output to terminal depending on no_color.
    """
    if no_color:
        click.echo(message)
    else:
        click.secho(message, fg=fg)


# -----------------------------------------------------------------------------
# Process shared options to all commands
def process_shared_options(context_obj, kwargs):
    """
    Update context with values for shared options.
    """
    context_obj['config_dir'] = kwargs.get('config_dir')
    context_obj['credentials_file'] = kwargs.get('credentials_file')
    context_obj['log_level'] = kwargs.get('log_level')
    context_obj['no_color'] = kwargs.get('no_color')
    context_obj['profile'] = kwargs.get('profile')
    context_obj['region'] = kwargs.get('region')
    context_obj['resource_group'] = kwargs.get('resource_group')
    context_obj['container'] = kwargs.get('container')
    context_obj['storage_account'] = kwargs.get('storage_account')
    context_obj['publisher_id'] = kwargs.get('publisher_id')
    context_obj['notification_emails'] = kwargs.get('notification_emails')


# -----------------------------------------------------------------------------
# Get dict from json file
def get_obj_from_json_file(json_file):
    j_file = os.path.expanduser(json_file)

    return_object = None

    with open(j_file, 'r') as my_json_file:
        return_object = json.load(my_json_file)

    return return_object



import pytest

from unittest.mock import MagicMock, patch

from azure_img_utils.auth import (
   acquire_access_token,
)

from azure_img_utils.exceptions import (
    AzureImgUtilsException
)


class TestAzureAuth(object):
    def setup_class(self):
        pass

    @patch('azure_img_utils.auth.msal.ConfidentialClientApplication')
    def test_acquire_access_token(self, mock_cclient_app):

        my_credentials = {
            'clientId': 'myClientId',
            'clientSecret': 'myClientSecret',
            'activeDirectoryEndpointUrl': 'myADEndpointUrl',
            'managementEndpointUrl': 'myMgmtEndpointUrl',
            'tenantId': 'myTenantId'
        }

        my_client = MagicMock()

        my_response = {
            'access_token': 'mySecretAccessToken'
        }

        my_client.acquire_token_for_client.return_value = my_response
        mock_cclient_app.return_value = my_client

        my_token = acquire_access_token(
            my_credentials,
            cloud_partner=False
        )
        assert my_token == 'mySecretAccessToken'

        # Error condidion
        my_err_response = {
            'access_token': 'mySecretAccessToken',
            'error': 'myCustomError'
        }

        my_client.acquire_token_for_client.return_value = my_err_response

        msg = 'Unable to authenticate against ' \
              'https://cloudpartner.azure.com/.default: ' \
              'myCustomError'

        with pytest.raises(AzureImgUtilsException, match=msg):
            my_token = acquire_access_token(
                my_credentials,
                cloud_partner=True
            )



#!/usr/bin/python3
# -*- coding: utf-8 -*-

"""Azure img utils cli unit tests."""

# Copyright (c) 2022 SUSE LLC. All rights reserved.
#
# This file is part of azure_img_utils. azure_img_utils provides an
# api and command line utilities for handling images in the Azure Cloud.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from unittest.mock import patch, MagicMock

from azure_img_utils.cli.cli import az_img_utils
from click.testing import CliRunner


def test_client_help():
    """Confirm azure img utils --help is successful."""
    runner = CliRunner()
    result = runner.invoke(az_img_utils, ['--help'])
    assert result.exit_code == 0
    assert 'The command line interface provides ' \
           'azure image utilities' in result.output


def test_print_license():
    runner = CliRunner()
    result = runner.invoke(az_img_utils, ['--license'])
    assert result.exit_code == 0
    assert result.output == 'GPLv3+\n'


# -------------------------------------------------
# authentication parameters tests
@patch('azure_img_utils.cli.blob.AzureImage')
def test_auth_provided_credentials_via_credentials_file(
    azure_image_mock
):
    """Confirm if authentication parameters are provided all is ok.
    """
    image_class = MagicMock()
    image_class.image_blob_exists.return_value = True
    azure_image_mock.return_value = image_class

    args = [
        'blob', 'exists',
        '--credentials-file', 'tests/creds.json',
        '--resource-group', 'myResourceGroup',
        '--storage-account', 'myStorageAccount',
        '--blob-name', 'myBlobName',
        '--container', 'myContainer'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 0


@patch('azure_img_utils.cli.blob.AzureImage')
def test_auth_provided_credentials_via_credentials_file_exception(
    azure_image_mock
):
    """Confirm if authentication parameters are provided all is ok.
    resource-group missing
    """

    def my_side_eff(*args, **kwargs):
        raise Exception('myException')

    image_class = MagicMock()
    image_class.image_blob_exists.side_effect = my_side_eff
    azure_image_mock.return_value = image_class

    args = [
        'blob', 'exists',
        '--credentials-file', 'tests/creds.json',
        '--storage-account', 'myStorageAccount',
        '--blob-name', 'myBlobName',
        '--container', 'myContainer'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 1
    assert "Unable to check blob existence" in result.output
    assert "myException" in result.output


# -------------------------------------------------
# parameter precedence tests
@patch('azure_img_utils.cli.blob.AzureImage')
def test_parameter_precedence(
    azure_image_mock
):
    """Confirm parameter precedence is OK
    """

    instance = MagicMock()
    instance.image_blob_exists.return_value = True

    def my_side_eff(*args, **kwargs):
        print("container->" + kwargs["container"])
        print("storage_account->" + kwargs["storage_account"])
        print("resource_group->" + kwargs["resource_group"])
        return instance

    azure_image_mock.side_effect = my_side_eff

    args = [
        'blob', 'exists',
        '--credentials-file', 'tests/creds.json',
        '--config-dir', './tests',
        '--profile', 'default2',
        '--container', 'myContainer',
        '--blob-name', 'myBlobName',
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 0
    assert "true" in result.output
    assert "container->myContainer" in result.output
    assert "resource_group->my_resource_group" in result.output
    assert "storage_account->my_storage_account" in result.output


# -------------------------------------------------
# unknown keyword in config file
@patch('azure_img_utils.cli.blob.AzureImage')
def test_unknown_keyword_in_config(azure_image_mock):
    """Confirm unknown keyword in config is handled ok"""
    image_class = MagicMock()
    image_class.image_blob_exists.return_value = False
    azure_image_mock.return_value = image_class

    args = [
        'blob', 'exists',
        '--credentials-file', 'tests/creds.json',
        '--profile', 'unknown_keyword',
        '--config-dir', 'tests',
        '--storage-account', 'myStorageAccount',
        '--blob-name', 'myBlobName',
        '--container', 'myContainer',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 1
    assert 'Found unknown keyword in config file' in result.output


# -------------------------------------------------
# blob exists
@patch('azure_img_utils.cli.blob.AzureImage')
def test_blob_exists_ok(azure_image_mock):
    """Confirm blob exists is ok"""
    image_class = MagicMock()
    image_class.image_blob_exists.return_value = False
    azure_image_mock.return_value = image_class

    args = [
        'blob', 'exists',
        '--credentials-file', 'tests/creds.json',
        '--storage-account', 'myStorageAccount',
        '--blob-name', 'myBlobName',
        '--container', 'myContainer',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 0
    assert 'false' in result.output


@patch('azure_img_utils.cli.blob.AzureImage')
def test_blob_exists_ok2(azure_image_mock):
    """Confirm blob exists is ok"""
    image_class = MagicMock()
    image_class.image_blob_exists.return_value = True
    azure_image_mock.return_value = image_class

    args = [
        'blob', 'exists',
        '--credentials-file', 'tests/creds.json',
        '--storage-account', 'myStorageAccount',
        '--blob-name', 'myBlobName',
        '--container', 'myContainer',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 0
    assert 'true' in result.output


@patch('azure_img_utils.cli.blob.AzureImage')
def test_blob_exists_nok_blobname_missing(azure_image_mock):
    """blob exists test with --blob-name missing"""
    image_class = MagicMock()
    image_class.image_blob_exists.return_value = False
    azure_image_mock.return_value = image_class

    args = [
        'blob', 'exists',
        '--credentials-file', 'tests/creds.json',
        '--storage-account', 'myStorageAccount',
        '--container', 'myContainer',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 2
    assert "Missing option '--blob-name'" in result.output


@patch('azure_img_utils.cli.blob.AzureImage')
def test_blob_exists_exception(azure_image_mock):
    """Confirm if exception handling is ok"""
    image_class = MagicMock()

    def my_side_eff(*args, **kwargs):
        raise Exception('myException')

    image_class.image_blob_exists.side_effect = my_side_eff
    azure_image_mock.return_value = image_class

    args = [
        'blob', 'exists',
        '--credentials-file', 'tests/creds.json',
        '--storage-account', 'myStorageAccount',
        '--blob-name', 'myBlobName',
        '--container', 'myContainer',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 1
    assert 'Unable to check blob existence' in result.output
    assert 'myException' in result.output


# -------------------------------------------------
# blob upload
@patch('azure_img_utils.cli.blob.AzureImage')
def test_blob_upload_ok(azure_image_mock):
    """Confirm blob upload is ok"""
    image_class = MagicMock()
    image_class.upload_image_blob.return_value = 'myBlobName'
    azure_image_mock.return_value = image_class

    args = [
        'blob', 'upload',
        '--credentials-file', 'tests/creds.json',
        '--storage-account', 'myStorageAccount',
        '--blob-name', 'myBlobName',
        '--container', 'myContainer',
        '--max-attempts', '3',
        '--image-file', 'tests/image.raw',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 0
    assert 'blob myBlobName uploaded' in result.output


@patch('azure_img_utils.cli.blob.AzureImage')
def test_blob_upload_nok_blobname_missing(azure_image_mock):
    """blob upload test with --blob-name missing"""
    image_class = MagicMock()
    image_class.upload_image_blob.return_value = 'myBlobName'
    azure_image_mock.return_value = image_class

    args = [
        'blob', 'upload',
        '--credentials-file', 'tests/creds.json',
        '--storage-account', 'myStorageAccount',
        '--container', 'myContainer',
        '--image-file', 'tests/image.raw',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 2
    assert "Missing option '--blob-name'" in result.output


@patch('azure_img_utils.cli.blob.AzureImage')
def test_blob_upload_nok_filename_missing(azure_image_mock):
    """blob upload test with --file-name missing"""
    image_class = MagicMock()
    image_class.upload_image_blob.return_value = 'myBlobName'
    azure_image_mock.return_value = image_class

    args = [
        'blob', 'upload',
        '--credentials-file', 'tests/creds.json',
        '--storage-account', 'myStorageAccount',
        '--blob-name', 'myBlobName',
        '--container', 'myContainer',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 2
    assert "Missing option '--image-file'" in result.output


@patch('azure_img_utils.cli.blob.AzureImage')
def test_blob_upload_nok_filename_notafile(azure_image_mock):
    """blob upload test with --file-name wrong"""
    image_class = MagicMock()
    image_class.upload_image_blob.return_value = 'myBlobName'
    azure_image_mock.return_value = image_class

    args = [
        'blob', 'upload',
        '--credentials-file', 'tests/creds.json',
        '--storage-account', 'myStorageAccount',
        '--blob-name', 'myBlobName',
        '--container', 'myContainer',
        '--image-file', 'tests/nonexistingfile.raw',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 2
    assert "Invalid value for '--image-file'" in result.output


@patch('azure_img_utils.cli.blob.AzureImage')
def test_blob_upload_exception(azure_image_mock):
    """Confirm if exception handling is ok"""

    def my_side_eff(*args, **kwargs):
        raise Exception('myException')

    image_class = MagicMock()
    image_class.upload_image_blob.side_effect = my_side_eff
    azure_image_mock.return_value = image_class

    args = [
        'blob', 'upload',
        '--credentials-file', 'tests/creds.json',
        '--storage-account', 'myStorageAccount',
        '--blob-name', 'myBlobName',
        '--container', 'myContainer',
        '--image-file', 'tests/image.raw',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 1
    assert 'Unable to upload blob' in result.output
    assert 'myException' in result.output


# -------------------------------------------------
# blob delete
@patch('azure_img_utils.cli.blob.AzureImage')
def test_blob_delete_ok(azure_image_mock):
    """Confirm blob delete is ok"""
    image_class = MagicMock()
    image_class.delete_storage_blob.return_value = True
    azure_image_mock.return_value = image_class

    args = [
        'blob', 'delete',
        '--credentials-file', 'tests/creds.json',
        '--storage-account', 'myStorageAccount',
        '--blob-name', 'myBlobName',
        '--container', 'myContainer',
        '--no-color',
        '--yes',
        '--verbose',
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 0
    assert 'blob deleted' in result.output


@patch('azure_img_utils.cli.blob.AzureImage')
def test_blob_delete_ok2(azure_image_mock):
    """Confirm blob delete is ok"""
    image_class = MagicMock()
    image_class.delete_storage_blob.return_value = False
    azure_image_mock.return_value = image_class

    args = [
        'blob', 'delete',
        '--credentials-file', 'tests/creds.json',
        '--storage-account', 'myStorageAccount',
        '--blob-name', 'myBlobName',
        '--container', 'myContainer',
        '--no-color',
        '--yes',
        '--verbose',
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 0
    assert 'blob myBlobName not found' in result.output


@patch('azure_img_utils.cli.blob.AzureImage')
def test_blob_delete_ok_confirmation(azure_image_mock):
    """Confirm blob delete is ok with confirmation"""
    image_class = MagicMock()
    image_class.delete_storage_blob.return_value = True
    azure_image_mock.return_value = image_class

    args = [
        'blob', 'delete',
        '--credentials-file', 'tests/creds.json',
        '--storage-account', 'myStorageAccount',
        '--blob-name', 'myBlobName',
        '--container', 'myContainer',
        '--no-color',
        '--verbose',
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args, input='y\n')
    assert result.exit_code == 0
    assert 'blob deleted' in result.output


@patch('azure_img_utils.cli.blob.AzureImage')
def test_blob_delete_ok_noconfirmation(azure_image_mock):
    """Confirm blob delete is ok with confirmation"""
    image_class = MagicMock()
    image_class.delete_storage_blob.return_value = True
    azure_image_mock.return_value = image_class

    args = [
        'blob', 'delete',
        '--credentials-file', 'tests/creds.json',
        '--storage-account', 'myStorageAccount',
        '--blob-name', 'myBlobName',
        '--container', 'myContainer',
        '--no-color',
        '--verbose',
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args, input='n\n')
    assert result.exit_code == 1
    assert 'Aborted' in result.output


@patch('azure_img_utils.cli.blob.AzureImage')
def test_blob_delete_nok_blobname_missing(azure_image_mock):
    """blob delete test with --blob-name missing"""
    image_class = MagicMock()
    image_class.delete_storage_blob.return_value = True
    azure_image_mock.return_value = image_class

    args = [
        'blob', 'delete',
        '--credentials-file', 'tests/creds.json',
        '--storage-account', 'myStorageAccount',
        '--container', 'myContainer',
        '--no-color',
        '--yes'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 2
    assert "Missing option '--blob-name'" in result.output


@patch('azure_img_utils.cli.blob.AzureImage')
def test_blob_delete_exception(azure_image_mock):
    """Confirm if exception handling is ok"""
    image_class = MagicMock()

    def my_side_eff(*args, **kwargs):
        raise Exception('myException')

    image_class.delete_storage_blob.side_effect = my_side_eff
    azure_image_mock.return_value = image_class

    args = [
        'blob', 'delete',
        '--credentials-file', 'tests/creds.json',
        '--storage-account', 'myStorageAccount',
        '--blob-name', 'myBlobName',
        '--container', 'myContainer',
        '--yes',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 1
    assert 'Unable to delete blob' in result.output
    assert 'myException' in result.output


# -------------------------------------------------
# image exists
@patch('azure_img_utils.cli.image.AzureImage')
def test_image_exists_ok_false(azure_image_mock):
    """Confirm image exists is ok"""
    image_class = MagicMock()
    image_class.image_exists.return_value = False
    azure_image_mock.return_value = image_class

    args = [
        'image', 'exists',
        '--credentials-file', 'tests/creds.json',
        '--storage-account', 'myStorageAccount',
        '--image-name', 'myBlobName',
        '--container', 'myContainer',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 0
    assert 'false' in result.output


@patch('azure_img_utils.cli.image.AzureImage')
def test_image_exists_ok_true(azure_image_mock):
    """Confirm image exists is ok"""
    image_class = MagicMock()
    image_class.image_blob_exists.return_value = True
    azure_image_mock.return_value = image_class

    args = [
        'image', 'exists',
        '--credentials-file', 'tests/creds.json',
        '--storage-account', 'myStorageAccount',
        '--image-name', 'myBlobName',
        '--container', 'myContainer',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 0
    assert 'true' in result.output


@patch('azure_img_utils.cli.image.AzureImage')
def test_image_exists_nok_image_name_missing(azure_image_mock):
    """image exists test with exception"""
    image_class = MagicMock()
    image_class.image_blob_exists.return_value = False
    azure_image_mock.return_value = image_class

    args = [
        'image', 'exists',
        '--credentials-file', 'tests/creds.json',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 2
    assert "Missing option '--image-name'" in result.output


@patch('azure_img_utils.cli.image.AzureImage')
def test_image_exists_nok_exc(azure_image_mock):
    """image exists test with some exception"""

    def my_side_eff(*args, **kwargs):
        raise Exception('myException')

    image_class = MagicMock()
    image_class.image_exists.side_effect = my_side_eff
    azure_image_mock.return_value = image_class

    args = [
        'image', 'exists',
        '--credentials-file', 'tests/creds.json',
        '--image-name', 'my_image_name',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 1
    assert "myException" in result.output


# -------------------------------------------------
# image create
@patch('azure_img_utils.cli.image.AzureImage')
def test_image_create_ok(azure_image_mock):
    """Confirm image create is ok"""
    image_class = MagicMock()
    image_class.create_compute_image.return_value = 'myImageName'
    azure_image_mock.return_value = image_class

    args = [
        'image', 'create',
        '--credentials-file', 'tests/creds.json',
        '--blob-name', 'myBlobName',
        '--image-name', 'myImageName',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 0
    assert 'image myImageName created' in result.output


@patch('azure_img_utils.cli.image.AzureImage')
def test_image_create_nok_blobname_missing(azure_image_mock):
    """blob upload test with --blob-name missing"""
    image_class = MagicMock()
    image_class.create_compute_image.return_value = 'myImageName'
    azure_image_mock.return_value = image_class

    args = [
        'image', 'create',
        '--credentials-file', 'tests/creds.json',
        '--image-name', 'myImageName',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 2
    assert "Missing option '--blob-name'" in result.output


@patch('azure_img_utils.cli.image.AzureImage')
def test_image_create_nok_imagename_missing(azure_image_mock):
    """blob upload test with --image-name missing"""
    image_class = MagicMock()
    image_class.create_compute_image.return_value = 'myImageName'
    azure_image_mock.return_value = image_class

    args = [
        'image', 'create',
        '--credentials-file', 'tests/creds.json',
        '--blob-name', 'myBlobName',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 2
    assert "Missing option '--image-name'" in result.output


@patch('azure_img_utils.cli.image.AzureImage')
def test_image_create_exception(azure_image_mock):
    """Confirm if exception handling is ok"""

    def my_side_eff(*args, **kwargs):
        raise Exception('myException')

    image_class = MagicMock()
    image_class.create_compute_image.side_effect = my_side_eff
    azure_image_mock.return_value = image_class

    args = [
        'image', 'create',
        '--credentials-file', 'tests/creds.json',
        '--config-dir', 'tests/',
        '--profile', 'default2',
        '--blob-name', 'myBlobName',
        '--image-name', 'myImageName',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 1
    assert 'Unable to create image' in result.output
    assert 'myException' in result.output


# -------------------------------------------------
# image delete
@patch('azure_img_utils.cli.image.AzureImage')
def test_image_delete_ok(azure_image_mock):
    """Confirm delete is ok"""
    image_class = MagicMock()
    image_class.delete_compute_image.return_value = None
    azure_image_mock.return_value = image_class

    args = [
        'image', 'delete',
        '--credentials-file', 'tests/creds.json',
        '--image-name', 'myImageName',
        '--no-color',
        '--yes',
        '--verbose',
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 0


@patch('azure_img_utils.cli.image.AzureImage')
def test_image_delete_ok_confirmation(azure_image_mock):
    """Confirm image delete is ok with confirmation"""
    image_class = MagicMock()
    image_class.delete_compute_image.return_value = None
    azure_image_mock.return_value = image_class

    args = [
        'image', 'delete',
        '--credentials-file', 'tests/creds.json',
        '--image-name', 'myImageName',
        '--no-color',
        '--verbose',
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args, input='y\n')
    assert result.exit_code == 0


@patch('azure_img_utils.cli.image.AzureImage')
def test_image_delete_ok_noconfirmation(azure_image_mock):
    """Confirm image delete is ok answering NO to confirmation"""
    image_class = MagicMock()
    image_class.delete_compute_image.return_value = None
    azure_image_mock.return_value = image_class

    args = [
        'image', 'delete',
        '--credentials-file', 'tests/creds.json',
        '--image-name', 'myImageName',
        '--no-color',
        '--verbose',
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args, input='n\n')
    assert result.exit_code == 1
    assert 'Aborted' in result.output


@patch('azure_img_utils.cli.image.AzureImage')
def test_image_delete_nok_exception(azure_image_mock):
    """image delete test with some exception"""

    def my_side_eff(*args, **kwargs):
        raise Exception('myException')

    image_class = MagicMock()
    image_class.delete_compute_image.side_effect = my_side_eff
    azure_image_mock.return_value = image_class

    args = [
        'image', 'delete',
        '--credentials-file', 'tests/creds.json',
        '--image-name', 'myImageName',
        '--no-color',
        '--yes',
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 1
    assert "myException" in result.output


def test_image_delete_image_name_missing():
    """image delete without --image-name parameter"""

    args = [
        'image', 'delete',
        '--credentials-file', 'tests/creds.json',
        '--yes',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 2
    assert "Missing option '--image-name'" in result.output


# -------------------------------------------------
# gallery image version exists
@patch('azure_img_utils.cli.gallery_image_version.AzureImage')
def test_gallery_image_version_exists_ok_false(azure_image_mock):
    """Confirm gallery image version exists is ok. Image does not exist."""
    image_class = MagicMock()
    image_class.gallery_image_version_exists.return_value = False
    azure_image_mock.return_value = image_class

    args = [
        'gallery-image-version', 'exists',
        '--credentials-file', 'tests/creds.json',
        '--gallery-image-name', 'myImageName',
        '--gallery-name', 'myGalleryName',
        '--gallery-image-version', '0.0.1',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 0
    assert 'false' in result.output


@patch('azure_img_utils.cli.gallery_image_version.AzureImage')
def test_gallery_image_version_exists_ok_true(azure_image_mock):
    """Confirm gallery image version exists is ok. Image exists."""
    image_class = MagicMock()
    image_class.gallery_image_version_exists.return_value = True
    azure_image_mock.return_value = image_class

    args = [
        'gallery-image-version', 'exists',
        '--credentials-file', 'tests/creds.json',
        '--gallery-image-name', 'myImageName',
        '--gallery-name', 'myGalleryName',
        '--gallery-image-version', '0.0.1',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 0
    assert 'true' in result.output


@patch('azure_img_utils.cli.gallery_image_version.AzureImage')
def test_gallery_image_version_exists_nok_image_name_missing(azure_image_mock):
    """Gallery image version exists test with exception. No
     --gallery-image-name provided.
    """
    image_class = MagicMock()
    image_class.gallery_image_version_exists.return_value = True
    azure_image_mock.return_value = image_class

    args = [
        'gallery-image-version', 'exists',
        '--credentials-file', 'tests/creds.json',
        '--gallery-name', 'myGalleryName',
        '--gallery-image-version', '0.0.1',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 2
    assert "Missing option '--gallery-image-name'" in result.output


@patch('azure_img_utils.cli.gallery_image_version.AzureImage')
def test_gallery_image_version_exists_nok_gallery_name_missing(
    azure_image_mock
):
    """Gallery image version exists test with exception. No
     --gallery-name provided
     """
    image_class = MagicMock()
    image_class.gallery_image_version_exists.return_value = True
    azure_image_mock.return_value = image_class

    args = [
        'gallery-image-version', 'exists',
        '--credentials-file', 'tests/creds.json',
        '--gallery-image-name', 'myImageName',
        '--gallery-image-version', '0.0.1',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 2
    assert "Missing option '--gallery-name'" in result.output


@patch('azure_img_utils.cli.gallery_image_version.AzureImage')
def test_gallery_image_version_exists_nok_image_version_missing(
    azure_image_mock
):
    """Gallery image version exists test with exception. No
     --gallery-image-version provided
    """
    image_class = MagicMock()
    image_class.gallery_image_version_exists.return_value = True
    azure_image_mock.return_value = image_class

    args = [
        'gallery-image-version', 'exists',
        '--credentials-file', 'tests/creds.json',
        '--gallery-image-name', 'myImageName',
        '--gallery-name', 'myGalleryName',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 2
    assert "Missing option '--gallery-image-version'" in result.output


@patch('azure_img_utils.cli.gallery_image_version.AzureImage')
def test_gallery_image_version_exists_nok_exc(azure_image_mock):
    """Gallery image version exists test with some exception"""

    def my_side_eff(*args, **kwargs):
        raise Exception('myException')

    image_class = MagicMock()
    image_class.gallery_image_version_exists.side_effect = my_side_eff
    azure_image_mock.return_value = image_class

    args = [
        'gallery-image-version', 'exists',
        '--credentials-file', 'tests/creds.json',
        '--gallery-image-name', 'myImageName',
        '--gallery-name', 'myGalleryName',
        '--gallery-image-version', '0.0.1',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 1
    assert "myException" in result.output


# -------------------------------------------------
# gallery image version create
@patch('azure_img_utils.cli.gallery_image_version.AzureImage')
def test_gallery_image_version_create_ok(azure_image_mock):
    """Confirm gallery image version create is ok."""
    image_class = MagicMock()
    image_class.create_gallery_image_version.return_value = 'myImageName'
    azure_image_mock.return_value = image_class

    args = [
        'gallery-image-version', 'create',
        '--credentials-file', 'tests/creds.json',
        '--blob-name', 'myBlobName',
        '--gallery-name', 'myGalleryName',
        '--gallery-image-name', 'myImageName',
        '--gallery-image-version', '0.0.1',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    print("RES->"+result.output)
    assert result.exit_code == 0
    assert 'gallery image version myImageName created' in result.output


@patch('azure_img_utils.cli.gallery_image_version.AzureImage')
def test_gallery_image_version_create_image_name_missing(azure_image_mock):
    """Gallery image version create test. No --gallery-image-name provided"""
    image_class = MagicMock()
    image_class.create_gallery_image_version.return_value = 'myImageName'
    azure_image_mock.return_value = image_class

    args = [
        'gallery-image-version', 'create',
        '--credentials-file', 'tests/creds.json',
        '--blob-name', 'myBlobName',
        '--gallery-name', 'myGalleryName',
        '--gallery-image-version', '0.0.1',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 2
    assert "Missing option '--gallery-image-name'" in result.output


@patch('azure_img_utils.cli.gallery_image_version.AzureImage')
def test_gallery_image_version_create_blob_name_missing(azure_image_mock):
    """Gallery image version create test. No --blob-name provided"""
    image_class = MagicMock()
    image_class.create_gallery_image_version.return_value = 'myImageName'
    azure_image_mock.return_value = image_class

    args = [
        'gallery-image-version', 'create',
        '--credentials-file', 'tests/creds.json',
        '--gallery-name', 'myGalleryName',
        '--gallery-image-name', 'myImageName',
        '--gallery-image-version', '0.0.1',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 2
    assert "Missing option '--blob-name'" in result.output


@patch('azure_img_utils.cli.gallery_image_version.AzureImage')
def test_gallery_image_version_create_gallery_name_missing(azure_image_mock):
    """Gallery image version create test. No --gallery-name provided"""
    image_class = MagicMock()
    image_class.create_gallery_image_version.return_value = 'myImageName'
    azure_image_mock.return_value = image_class

    args = [
        'gallery-image-version', 'create',
        '--credentials-file', 'tests/creds.json',
        '--blob-name', 'myBlobName',
        '--gallery-image-name', 'myImageName',
        '--gallery-image-version', '0.0.1',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 2
    assert "Missing option '--gallery-name'" in result.output


@patch('azure_img_utils.cli.gallery_image_version.AzureImage')
def test_gallery_image_version_create_image_version_missing(azure_image_mock):
    """Gallery image version create test. No --gallery-image-version
     provided
    """
    image_class = MagicMock()
    image_class.create_gallery_image_version.return_value = 'myImageName'
    azure_image_mock.return_value = image_class

    args = [
        'gallery-image-version', 'create',
        '--credentials-file', 'tests/creds.json',
        '--blob-name', 'myBlobName',
        '--gallery-image-name', 'myImageName',
        '--gallery-name', 'myGalleryName',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 2
    assert "Missing option '--gallery-image-version'" in result.output


@patch('azure_img_utils.cli.gallery_image_version.AzureImage')
def test_gallery_image_version_create_nok_exc(azure_image_mock):
    """Gallery image version create test with some exception"""

    def my_side_eff(*args, **kwargs):
        raise Exception('myException')

    image_class = MagicMock()
    image_class.create_gallery_image_version.side_effect = my_side_eff
    azure_image_mock.return_value = image_class

    args = [
        'gallery-image-version', 'create',
        '--credentials-file', 'tests/creds.json',
        '--gallery-image-name', 'myImageName',
        '--blob-name', 'myBlobName',
        '--gallery-name', 'myGalleryName',
        '--gallery-image-version', '0.0.1',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 1
    assert "myException" in result.output


# -------------------------------------------------
# gallery image version delete
@patch('azure_img_utils.cli.gallery_image_version.AzureImage')
def test_gallery_image_version_delete_ok(azure_image_mock):
    """Confirm gallery image version delete is ok."""
    image_class = MagicMock()
    image_class.delete_gallery_image_version.return_value = None
    azure_image_mock.return_value = image_class

    args = [
        'gallery-image-version', 'delete',
        '--credentials-file', 'tests/creds.json',
        '--gallery-image-name', 'myImageName',
        '--gallery-name', 'myGalleryName',
        '--gallery-image-version', '0.0.1',
        '--yes',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 0


@patch('azure_img_utils.cli.gallery_image_version.AzureImage')
def test_gallery_image_version_delete_nok_image_name_missing(azure_image_mock):
    """Gallery image version delete test. No --gallery-image-name provided"""
    image_class = MagicMock()
    image_class.delete_gallery_image_version.return_value = None
    azure_image_mock.return_value = image_class

    args = [
        'gallery-image-version', 'delete',
        '--credentials-file', 'tests/creds.json',
        '--gallery-name', 'myGalleryName',
        '--gallery-image-version', '0.0.1',
        '--yes',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 2
    assert "Missing option '--gallery-image-name'" in result.output


@patch('azure_img_utils.cli.gallery_image_version.AzureImage')
def test_gallery_image_version_delete_nok_gallery_name_missing(
    azure_image_mock
):
    """Gallery image version delete test. No --gallery-name provided"""
    image_class = MagicMock()
    image_class.delete_gallery_image_version.return_value = None
    azure_image_mock.return_value = image_class

    args = [
        'gallery-image-version', 'delete',
        '--credentials-file', 'tests/creds.json',
        '--gallery-image-name', 'myImageName',
        '--gallery-image-version', '0.0.1',
        '--yes',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 2
    assert "Missing option '--gallery-name'" in result.output


@patch('azure_img_utils.cli.gallery_image_version.AzureImage')
def test_gallery_image_version_delete_nok_image_version_missing(
    azure_image_mock
):
    """Gallery image version delete test. No --gallery-image-version provided
    """
    image_class = MagicMock()
    image_class.delete_gallery_image_version.return_value = None
    azure_image_mock.return_value = image_class

    args = [
        'gallery-image-version', 'delete',
        '--credentials-file', 'tests/creds.json',
        '--gallery-image-name', 'myImageName',
        '--gallery-name', 'myGalleryName',
        '--yes',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 2
    assert "Missing option '--gallery-image-version'" in result.output


@patch('azure_img_utils.cli.gallery_image_version.AzureImage')
def test_gallery_image_version_delete_nok_exc(azure_image_mock):
    """Gallery image version delete test with some exception"""

    def my_side_eff(*args, **kwargs):
        raise Exception('myException')

    image_class = MagicMock()
    image_class.delete_gallery_image_version.side_effect = my_side_eff
    azure_image_mock.return_value = image_class

    args = [
        'gallery-image-version', 'delete',
        '--credentials-file', 'tests/creds.json',
        '--gallery-image-name', 'myImageName',
        '--gallery-name', 'myGalleryName',
        '--gallery-image-version', '0.0.1',
        '--yes',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 1
    assert "myException" in result.output
    assert "Unable to delete gallery image version" in result.output


# -------------------------------------------------
# cloud-partner-offer publish tests
@patch('azure_img_utils.cli.offer.AzureImage')
def test_cloud_partner_offer_publish_ok(azure_image_mock):
    """Confirm cloud partner offer publish is ok."""

    myUrl = "https://mytest.com/locationURL"
    image_class = MagicMock()
    image_class.publish_offer.return_value = myUrl
    azure_image_mock.return_value = image_class

    args = [
        'cloud-partner-offer', 'publish',
        '--credentials-file', 'tests/creds.json',
        '--offer-id', 'myOfferId',
        '--publisher-id', 'myPublisherId',
        '--notification-emails', '1@mail.com,2@mail.com',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 0
    assert "Published cloud partner offer." in result.output
    assert "Operation URI: " + myUrl in result.output


@patch('azure_img_utils.cli.offer.AzureImage')
def test_cloud_partner_offer_publish_offer_id_not_provided(
    azure_image_mock
):
    """Cloud partner offer publish nok. --offer-id not provided"""

    myUrl = "https://mytest.com/locationURL"
    image_class = MagicMock()
    image_class.publish_offer.return_value = myUrl
    azure_image_mock.return_value = image_class

    args = [
        'cloud-partner-offer', 'publish',
        '--credentials-file', 'tests/creds.json',
        '--publisher-id', 'myPublisherId',
        '--notification-emails', '1@mail.com,2@mail.com',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 2
    assert "Missing option '--offer-id'" in result.output


@patch('azure_img_utils.cli.offer.AzureImage')
def test_cloud_partner_offer_publish_exc(azure_image_mock):
    """Confirm cloud partner offer publish exception handling is ok."""

    def my_side_eff(*args, **kwargs):
        raise Exception('myException')

    image_class = MagicMock()
    image_class.publish_offer.side_effect = my_side_eff
    azure_image_mock.return_value = image_class

    args = [
        'cloud-partner-offer', 'publish',
        '--credentials-file', 'tests/creds.json',
        '--offer-id', 'myOfferId',
        '--publisher-id', 'myPublisherId',
        '--notification-emails', '1@mail.com,2@mail.com',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 1
    assert "Unable to publish cloud partner offer" in result.output
    assert "myException" in result.output


# -------------------------------------------------
# cloud-partner-offer publish tests
@patch('azure_img_utils.cli.offer.AzureImage')
def test_cloud_partner_offer_go_live_ok(azure_image_mock):
    """Confirm cloud partner offer go-live is ok."""

    myUrl = "https://mytest.com/locationURL"
    image_class = MagicMock()
    image_class.go_live_with_offer.return_value = myUrl
    azure_image_mock.return_value = image_class

    args = [
        'cloud-partner-offer', 'go-live',
        '--credentials-file', 'tests/creds.json',
        '--offer-id', 'myOfferId',
        '--publisher-id', 'myPublisherId',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 0
    assert "Cloud partner offer set as go-live." in result.output
    assert "Operation URI: " + myUrl in result.output


@patch('azure_img_utils.cli.offer.AzureImage')
def test_cloud_partner_offer_go_live_offer_id_not_provided(
    azure_image_mock
):
    """Cloud partner offer go-live nok. --offer-id not provided"""

    myUrl = "https://mytest.com/locationURL"
    image_class = MagicMock()
    image_class.go_live_with_offer.return_value = myUrl
    azure_image_mock.return_value = image_class

    args = [
        'cloud-partner-offer', 'go-live',
        '--credentials-file', 'tests/creds.json',
        '--publisher-id', 'myPublisherId',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 2
    assert "Missing option '--offer-id'" in result.output


@patch('azure_img_utils.cli.offer.AzureImage')
def test_cloud_partner_offer_go_live_exc(azure_image_mock):
    """Confirm cloud partner offer go_live exception handling is ok."""

    def my_side_eff(*args, **kwargs):
        raise Exception('myException')

    image_class = MagicMock()
    image_class.go_live_with_offer.side_effect = my_side_eff
    azure_image_mock.return_value = image_class

    args = [
        'cloud-partner-offer', 'go-live',
        '--credentials-file', 'tests/creds.json',
        '--offer-id', 'myOfferId',
        '--publisher-id', 'myPublisherId',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 1
    assert "Unable to set cloud partner offer as go-live." in result.output
    assert "myException" in result.output


# -------------------------------------------------
# cloud-partner-offer upload-offer-document tests
@patch('azure_img_utils.cli.offer.AzureImage')
def test_cloud_partner_offer_upload_doc_ok(azure_image_mock):
    """Confirm cloud partner offer upload-offer-document is ok."""

    image_class = MagicMock()
    azure_image_mock.return_value = image_class

    args = [
        'cloud-partner-offer', 'upload-offer-document',
        '--credentials-file', 'tests/creds.json',
        '--offer-id', 'myOfferId',
        '--publisher-id', 'myPublisherId',
        '--offer-document-file', 'tests/creds.json',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 0


@patch('azure_img_utils.cli.offer.AzureImage')
def test_cloud_partner_offer_upload_doc_offer_id_not_provided(
    azure_image_mock
):
    """Cloud partner offer upload-offer-document nok.
    --offer-id not provided
    """

    image_class = MagicMock()
    image_class.upload_offer_doc.return_value = None
    azure_image_mock.return_value = image_class

    args = [
        'cloud-partner-offer', 'upload-offer-document',
        '--credentials-file', 'tests/creds.json',
        '--publisher-id', 'myPublisherId',
        '--offer-document-file', 'tests/creds.json',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 2
    assert "Missing option '--offer-id'" in result.output


@patch('azure_img_utils.cli.offer.AzureImage')
def test_cloud_partner_offer_upload_doc_document_file_not_provided(
    azure_image_mock
):
    """Cloud partner offer upload-offer-document nok.
    --offer-document-file not provided
    """

    image_class = MagicMock()
    image_class.upload_offer_doc.return_value = None
    azure_image_mock.return_value = image_class

    args = [
        'cloud-partner-offer', 'upload-offer-document',
        '--credentials-file', 'tests/creds.json',
        '--offer-id', 'myOfferId',
        '--publisher-id', 'myPublisherId',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    print("Result"+result.output)
    assert result.exit_code == 2
    assert "Missing option '--offer-document-file'" in result.output


@patch('azure_img_utils.cli.offer.AzureImage')
def test_cloud_partner_offer_upload_doc_exc(azure_image_mock):
    """Cloud partner offer upload-offer-document nok.
    Exception
    """

    def my_side_eff(*args, **kwargs):
        raise Exception('myException')

    image_class = MagicMock()
    image_class.upload_offer_doc.side_effect = my_side_eff
    azure_image_mock.return_value = image_class

    args = [
        'cloud-partner-offer', 'upload-offer-document',
        '--credentials-file', 'tests/creds.json',
        '--offer-id', 'myOfferId',
        '--publisher-id', 'myPublisherId',
        '--offer-document-file', 'tests/creds.json',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    print("Result"+result.output)
    assert result.exit_code == 1
    assert "Unable to upload cloud partner offer document." in result.output
    assert "myException" in result.output


# -------------------------------------------------
# cloud-partner-offer add-image-to-offer tests
@patch('azure_img_utils.cli.offer.AzureImage')
def test_cloud_partner_offer_add_image_ok(azure_image_mock):
    """Confirm cloud partner offer add-image-to-offer is ok."""

    image_class = MagicMock()
    azure_image_mock.return_value = image_class

    args = [
        'cloud-partner-offer', 'add-image-to-offer',
        '--credentials-file', 'tests/creds.json',
        '--blob-name', 'myBlobName',
        '--image-name', 'myImageName',
        '--image-description', 'my image description',
        '--offer-id', 'myOfferId',
        '--publisher-id', 'myPublisherId',
        '--label', 'myLabel',
        '--sku', 'mySku',
        '--blob-url', 'myBlobUrl',
        '--generation-id', 'V1',
        '--generation-suffix', 'mySuffix',
        '--vm-images-key', 'myKey',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 0


@patch('azure_img_utils.cli.offer.AzureImage')
def test_cloud_partner_offer_add_image_nok_blob_name_missing(azure_image_mock):
    """Confirm cloud partner offer add-image-to-offer handles params well.
    --blob-name missing"""

    image_class = MagicMock()
    azure_image_mock.return_value = image_class

    args = [
        'cloud-partner-offer', 'add-image-to-offer',
        '--credentials-file', 'tests/creds.json',
        '--image-name', 'myImageName',
        '--image-description', 'my image description',
        '--offer-id', 'myOfferId',
        '--publisher-id', 'myPublisherId',
        '--label', 'myLabel',
        '--sku', 'mySku',
        '--blob-url', 'myBlobUrl',
        '--generation-id', 'V1',
        '--generation-suffix', 'mySuffix',
        '--vm-images-key', 'myKey',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 2
    assert "Missing option '--blob-name'" in result.output


@patch('azure_img_utils.cli.offer.AzureImage')
def test_cloud_partner_offer_add_image_nok_image_name_missing(
    azure_image_mock
):
    """Confirm cloud partner offer add-image-to-offer handles params well.
    --image-name missing"""

    image_class = MagicMock()
    azure_image_mock.return_value = image_class

    args = [
        'cloud-partner-offer', 'add-image-to-offer',
        '--credentials-file', 'tests/creds.json',
        '--blob-name', 'myBlobName',
        '--image-description', 'my image description',
        '--offer-id', 'myOfferId',
        '--publisher-id', 'myPublisherId',
        '--label', 'myLabel',
        '--sku', 'mySku',
        '--blob-url', 'myBlobUrl',
        '--generation-id', 'V1',
        '--generation-suffix', 'mySuffix',
        '--vm-images-key', 'myKey',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 2
    assert "Missing option '--image-name'" in result.output


@patch('azure_img_utils.cli.offer.AzureImage')
def test_cloud_partner_offer_add_image_nok_image_description_missing(
    azure_image_mock
):
    """Confirm cloud partner offer add-image-to-offer handles params well.
    --image-description missing"""

    image_class = MagicMock()
    azure_image_mock.return_value = image_class

    args = [
        'cloud-partner-offer', 'add-image-to-offer',
        '--credentials-file', 'tests/creds.json',
        '--blob-name', 'myBlobName',
        '--image-name', 'myImageName',
        '--offer-id', 'myOfferId',
        '--publisher-id', 'myPublisherId',
        '--label', 'myLabel',
        '--sku', 'mySku',
        '--blob-url', 'myBlobUrl',
        '--generation-id', 'V1',
        '--generation-suffix', 'mySuffix',
        '--vm-images-key', 'myKey',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 2
    assert "Missing option '--image-description'" in result.output


@patch('azure_img_utils.cli.offer.AzureImage')
def test_cloud_partner_offer_add_image_nok_offer_id_missing(
    azure_image_mock
):
    """Confirm cloud partner offer add-image-to-offer handles params well.
    --offer-id missing"""

    image_class = MagicMock()
    azure_image_mock.return_value = image_class

    args = [
        'cloud-partner-offer', 'add-image-to-offer',
        '--credentials-file', 'tests/creds.json',
        '--blob-name', 'myBlobName',
        '--image-name', 'myImageName',
        '--image-description', 'my image description',
        '--publisher-id', 'myPublisherId',
        '--label', 'myLabel',
        '--sku', 'mySku',
        '--blob-url', 'myBlobUrl',
        '--generation-id', 'V1',
        '--generation-suffix', 'mySuffix',
        '--vm-images-key', 'myKey',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 2
    assert "Missing option '--offer-id'" in result.output


@patch('azure_img_utils.cli.offer.AzureImage')
def test_cloud_partner_offer_add_image_nok_label_missing(
    azure_image_mock
):
    """Confirm cloud partner offer add-image-to-offer handles params well.
    --label missing"""

    image_class = MagicMock()
    azure_image_mock.return_value = image_class

    args = [
        'cloud-partner-offer', 'add-image-to-offer',
        '--credentials-file', 'tests/creds.json',
        '--blob-name', 'myBlobName',
        '--image-name', 'myImageName',
        '--image-description', 'my image description',
        '--offer-id', 'myOfferId',
        '--publisher-id', 'myPublisherId',
        '--sku', 'mySku',
        '--blob-url', 'myBlobUrl',
        '--generation-id', 'V1',
        '--generation-suffix', 'mySuffix',
        '--vm-images-key', 'myKey',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 2
    assert "Missing option '--label'" in result.output


@patch('azure_img_utils.cli.offer.AzureImage')
def test_cloud_partner_offer_add_image_nok_sku_missing(
    azure_image_mock
):
    """Confirm cloud partner offer add-image-to-offer handles params well.
    --sku missing"""

    image_class = MagicMock()
    azure_image_mock.return_value = image_class

    args = [
        'cloud-partner-offer', 'add-image-to-offer',
        '--credentials-file', 'tests/creds.json',
        '--blob-name', 'myBlobName',
        '--image-name', 'myImageName',
        '--image-description', 'my image description',
        '--offer-id', 'myOfferId',
        '--publisher-id', 'myPublisherId',
        '--label', 'myLabel',
        '--blob-url', 'myBlobUrl',
        '--generation-id', 'V1',
        '--generation-suffix', 'mySuffix',
        '--vm-images-key', 'myKey',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 2
    assert "Missing option '--sku'" in result.output


@patch('azure_img_utils.cli.offer.AzureImage')
def test_cloud_partner_offer_add_image_nok_exc(
    azure_image_mock
):
    """Confirm cloud partner offer add-image-to-offer handles params well.
    exception"""

    def my_side_eff(*args, **kwargs):
        raise Exception('myException')

    image_class = MagicMock()
    image_class.add_image_to_offer.side_effect = my_side_eff
    azure_image_mock.return_value = image_class

    args = [
        'cloud-partner-offer', 'add-image-to-offer',
        '--credentials-file', 'tests/creds.json',
        '--blob-name', 'myBlobName',
        '--image-name', 'myImageName',
        '--image-description', 'my image description',
        '--offer-id', 'myOfferId',
        '--publisher-id', 'myPublisherId',
        '--label', 'myLabel',
        '--sku', 'mySku',
        '--blob-url', 'myBlobUrl',
        '--generation-id', 'V1',
        '--generation-suffix', 'mySuffix',
        '--vm-images-key', 'myKey',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 1
    assert "Unable to add image to cloud partner offer." in result.output
    assert "myException" in result.output


# -------------------------------------------------
# cloud-partner-offer remove-image-from-offer tests
@patch('azure_img_utils.cli.offer.AzureImage')
def test_cloud_partner_offer_remove_image_ok(azure_image_mock):
    """Confirm cloud partner offer remove-image-from-offer is ok."""

    image_class = MagicMock()
    azure_image_mock.return_value = image_class

    args = [
        'cloud-partner-offer', 'remove-image-from-offer',
        '--credentials-file', 'tests/creds.json',
        '--image-urn', 'myImageUrn',
        '--vm-images-key', 'myKey',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 0


@patch('azure_img_utils.cli.offer.AzureImage')
def test_cloud_partner_offer_remove_image_nok_image_urn_missing(
    azure_image_mock
):
    """Confirm cloud partner offer remove-image-from-offer handles
    params ok. --image-urn missing"""

    image_class = MagicMock()
    azure_image_mock.return_value = image_class

    args = [
        'cloud-partner-offer', 'remove-image-from-offer',
        '--credentials-file', 'tests/creds.json',
        '--vm-images-key', 'myKey',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 2
    assert "Missing option '--image-urn'" in result.output


@patch('azure_img_utils.cli.offer.AzureImage')
def test_cloud_partner_offer_remove_image_nok_exc(azure_image_mock):
    """Confirm cloud partner offer remove-image-from-offer handles
    exceptions ok."""

    def my_side_eff(*args, **kwargs):
        raise Exception('myException')

    image_class = MagicMock()
    image_class.remove_image_from_offer.side_effect = my_side_eff
    azure_image_mock.return_value = image_class

    args = [
        'cloud-partner-offer', 'remove-image-from-offer',
        '--credentials-file', 'tests/creds.json',
        '--image-urn', 'myImageUrn',
        '--vm-images-key', 'myKey',
        '--no-color'
    ]

    runner = CliRunner()
    result = runner.invoke(az_img_utils, args)
    assert result.exit_code == 1
    assert "myException" in result.output
    assert "Unable to remove image from cloud partner offer." in result.output



# -*- coding: utf-8 -*-

"""Azure image utils cli module."""

# Copyright (c) 2022 SUSE LLC
#
# This file is part of azure_img_utils. azure_img_utils provides an
# api and command line utilities for handling images in the Azure Cloud.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import click

from azure_img_utils.cli.blob import blob
from azure_img_utils.cli.image import image
from azure_img_utils.cli.gallery_image_version import (
    gallery_image_version
)
from azure_img_utils.cli.offer import offer


# -----------------------------------------------------------------------------
# license function
def print_license(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    click.echo('GPLv3+')
    ctx.exit()


# -----------------------------------------------------------------------------
# Main function
@click.group()
@click.version_option()
@click.option(
    '--license',
    is_flag=True,
    callback=print_license,
    expose_value=False,
    is_eager=True,
    help='Show license information.'
)
@click.pass_context
def az_img_utils(context):
    """
    The command line interface provides azure image utilities.

    This includes uploading image tarballs and
    creating/publishing/deprecating framework images.
    """
    if context.obj is None:
        context.obj = {}
    pass


az_img_utils.add_command(blob)
az_img_utils.add_command(image)
az_img_utils.add_command(gallery_image_version)
az_img_utils.add_command(offer)



# Copyright (c) 2021 SUSE LLC
#
# This file is part of azure_img_utils. azure_img_utils provides an
# api and command line utilities for handling images in the Azure Cloud.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from functools import singledispatch

from azure.mgmt.storage import StorageManagementClient
from azure.storage.blob import (
    BlobServiceClient,
    ContainerSasPermissions
)

from azure_img_utils.auth import create_sas_token, get_client_from_json
from azure_img_utils.exceptions import AzureImgUtilsStorageException


def get_blob_url(
    blob_service_client,
    blob_name: str,
    storage_account: str,
    container: str,
    permissions=ContainerSasPermissions(read=True, list=True),
    expire_hours: int = 1,
    start_hours: int = 1
):
    """
    Create a URL for the given blob with a shared access signature.

    The signature will expire based on expire_hours.
    """
    sas_token = create_sas_token(
        blob_service_client,
        storage_account,
        container,
        permissions=permissions,
        expire_hours=expire_hours,
        start_hours=start_hours
    )

    source_blob_url = (
        'https://{account}.blob.core.windows.net/'
        '{container}/{blob}?{token}'.format(
            account=storage_account,
            container=container,
            blob=blob_name,
            token=sas_token
        )
    )

    return source_blob_url


def get_storage_account_key(
    credentials: dict,
    resource_group: str,
    storage_account: str
):
    """Return the first storage account key for the provided account."""
    storage_client = get_client_from_json(
        StorageManagementClient,
        credentials
    )
    storage_key_list = storage_client.storage_accounts.list_keys(
        resource_group,
        storage_account
    )
    return storage_key_list.keys[0].value


def get_blob_client(blob_service_client, blob_name: str, container: str):
    """Return blob client based on container and blob name."""
    try:
        container_client = blob_service_client.get_container_client(container)
        blob_client = container_client.get_blob_client(blob_name)
    except ValueError as error:
        raise AzureImgUtilsStorageException(error) from error

    return blob_client


@singledispatch
def get_blob_service(
    credentials: dict,
    resource_group: str,
    storage_account: str
):
    """
    Return authenticated blob service instance for the storage account.

    Using storage account keys.
    """
    account_key = get_storage_account_key(
        credentials,
        resource_group,
        storage_account
    )

    return BlobServiceClient(
        account_url='https://{account_name}.blob.core.windows.net'.format(
            account_name=storage_account
        ),
        credential=account_key
    )


@get_blob_service.register(str)
def _(sas_token: str, storage_account: str):
    """
    Return authenticated page blob service instance for the storage account.

    Using an sas token.
    """
    return BlobServiceClient(
        account_url='https://{account_name}.blob.core.windows.net'.format(
            account_name=storage_account
        ),
        credential=sas_token
    )
